;**** Processor Definition
.include "tn12def.inc"

.equ	ninit	= 2											; number of ignigions per r
.equ	rpmlow	= 4000										; switch on rpm for low rpm led
.equ	rpmml	= 4500										; switch on rpm for mid rpm led
.equ	rpmmh	= 6000										; blink rpm for mid rpm led
.equ	rpmhigh	= 6500										; switch on rpm for high rpm led

;*****************************************************************************************
;	Variablen
;*****************************************************************************************

.def		Temp		= r16							; temporary register
.def		Timrec1		= r17							; Timer overflow counter Register
.def		Timrec2		= r18
.def		Status1		= r19							; led status register
.def		Tempport	= r20							; register to set up the output port
.def		Blinkreg	= r21							; register to blink the green led
.def		lcnt		= r22							; counters to eliminate blinking leds
.def		mlcnt		= r23
.def		mhcnt		= r24
.def		hcnt		= r25

;*****************************************************************************************
;	SRAM-Variablen
;*****************************************************************************************

;.equ	Nibble12		= 0x60			; 1st and 2nd caliper nibble


;*****************************************************************************************
;	Werte
;*****************************************************************************************

.equ	fovf	= 20000									; timer overflow frequency
.equ	Limitl 	= LOW(60*fovf/ninit/rpmlow)				; limit for low rpm
.equ	Limitlh	= HIGH(60*fovf/ninit/rpmlow)
.equ	Limitml	= LOW(60*fovf/ninit/rpmml)				; limit for mean low rpm
.equ	Limitmlh = HIGH(60*fovf/ninit/rpmml)
.equ	Limitmh	= LOW(60*fovf/ninit/rpmmh)				; limit for mean high rpm
.equ	Limitmhh = HIGH(60*fovf/ninit/rpmmh)
.equ	Limith	= LOW(60*fovf/ninit/rpmhigh)			; limit for high rpm
.equ	Limithh	= HIGH(60*fovf/ninit/rpmhigh)

;*****************************************************************************************
;	Programmcode
;*****************************************************************************************
.cseg
.org	$0000

;*****************************************************************************************
;	Interruptroutinen
;*****************************************************************************************

		rjmp	START					; reset vector
		rjmp	INT0ROUTINE				; Int0 Pin
		reti;	PINCHGROUTINE			; IO Pin changed
		rjmp	TIM0OVFROUTINE			; Timer/Counter0 Overflow
		reti;	EEREADYROUTINE			; EEPROM ready
		reti;	ANACOMPROUTINE			; analog comparator event

;*****************************************************************************************
;	Programmstart
;*****************************************************************************************

; clock is 1.2 MHz

START:
		clr		Temp									; set all pins low
		out		PORTB, Temp
		sbi		DDRB, 0
		sbi		DDRB, 3
		sbi		DDRB, 4
		ldi		Temp, 0b01000010						; set interrupt to falling edge
		out		MCUCR, Temp
		ldi		Temp, 0b01000000						; enable int0
		out		GIMSK, Temp
		ldi		Temp, 0b00000010						; enable timer 0 ovf int
		out		TIMSK, Temp
		ldi		Temp, 0b00000001						; set timer prescaler to 1
		out		TCCR0, Temp								; 1.2MHz/256 is the overflow frequency
		ldi		Temp, 250								; set timer overflow rate to 20kHz
		out		TCNT0, Temp
		clr		Timrec1
		clr		Timrec2
		sei												; enable interrupts

MAIN:
		sbrc	Status1, 4
		rjmp	SET_PORTS
		rjmp	MAIN

;*****************************************************************************************
;	Interruptroutinen
;***************************************************************************************** 

INT0ROUTINE:										; interrupt pin 0
		inc		Blinkreg
		andi	Status1, 0b00000000						; clear all leds
		cpi		Timrec2, Limitlh
		brne	INT0_LOW_NO_LTEST
		cpi		Timrec1, Limitl						; test if lowest limit achieved
 INT0_LOW_NO_LTEST:
		brlo	INT0_LOW_OFF
		ori		Status1, 0b00000001						; switch low rpm led on
 INT0_LOW_OFF:
		cpi		Timrec2, Limitmhh
		brne	INT0_MH_NO_LTEST
		cpi		Timrec1, Limitmh
 INT0_MH_NO_LTEST:
		brsh	INT0_MH_OFF
		ori		Status1, 0b00000100						; middle led blink state
		rjmp	INT0_ML_OFF
 INT0_MH_OFF:
		cpi		Timrec2, Limitmlh
		brne	INT0_ML_NO_LTEST
		cpi		Timrec1, Limitml
 INT0_ML_NO_LTEST:
		brsh	INT0_ML_OFF								; middle led off
		ori		Status1, 0b00000010						; middle led on
 INT0_ML_OFF:
		cpi		Timrec2, Limithh
		brne	INT0_H_NO_LTEST
		cpi		Timrec1, Limith
 INT0_H_NO_LTEST:
		brsh	INT0_H_OFF
		ori		Status1, 0b00001000						; high led on
		andi	Status1, 0b11111000						; all other leds off
 INT0_H_OFF:
 		ori		Status1, 0b00010000						; set led change activation state
		clr		Timrec1									; clear the timer ovf counter register
		clr		Timrec2
		reti

TIM0OVFROUTINE:										; timer0 overflow
		ldi		Temp, 1
		add		Timrec1, Temp							; increase the timer ovf counter register
		clr		Temp
		adc		Timrec2, Temp							; 16 bit are required
		ldi		Temp, 214								; set overflow frequency to 20kHz
		out		TCNT0, Temp
		brcc	TIM0OVFROUTINE_OUT
		clr		Temp									; clear all LEDs on timer overflow
		out		PORTB, Temp
 TIM0OVFROUTINE_OUT:
		reti

;*****************************************************************************************
;	Subroutinen
;*****************************************************************************************

SET_PORTS:											; switch on/off leds depending on Status1
		clr		Tempport
		sbrc	Status1, 0
		ori		Tempport, 0b00000001					; low rpm led
		sbrc	Status1, 1
		ori		Tempport, 0b00010000					; mid rpm led
		sbrs	Status1, 2
		rjmp	SET_PORTS_GO_ON
		sbrc	Blinkreg, 3
		ori		Tempport, 0b00010000
 SET_PORTS_GO_ON:
		sbrc	Status1, 3
		ori		Tempport, 0b00001000					; high rpm led
		out		PORTB, Tempport							; write the port
		clr		Status1
		rjmp	MAIN